@@ -550,8 +550,12 @@ C<static method getcwd : mutable string ($buf : mutable string, $size : int);>
 
 Calls the L<getcwd|https://linux.die.net/man/2/getcwd> function and return its return value.
 
+The buffer $buf must be undef.
+
 Exceptions:
 
+The buffer $buf must be undef. Otherwise an exception is thrown.
+
 $size must be greater than or equal to 0. Otherwise an exception is thrown.
 
 If $buf is defined, $size must be greater than or equal to 0. Otherwise an exception is thrown.
@@ -564,8 +568,12 @@ C<static method _getdcwd : mutable string ($drive : int, $buffer : mutable strin
 
 Calls the L<_getdcwd|https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/getdcwd-wgetdcwd?view=msvc-170> function and return its return value.
 
+The buffer $buffer must be undef.
+
 Exceptions:
 
+The buffer $buffer must be undef. Otherwise an exception is thrown.
+
 $maxlen must be greater than 0. Otherwise an exception is thrown.
 
 $maxlen must be less than or equal to the lenght of $buffer. Otherwise an exception is thrown.
@@ -584,18 +592,6 @@ $path must be defined. Otherwise an exception is thrown.
 
 If the realpath function failed, an exception is thrown with C<eval_error_id> set to the basic type ID of the L<Error::System|SPVM::Error::System> class.
 
-=head2 _fullpath
-
-C<native static method _fullpath : mutable string ($absPath : mutable string, $relPath : string, $maxLength : int);>
-
-Calls the L<_fullpath|https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/fullpath-wfullpath?view=msvc-170> function and return its return value.
-
-Exceptions:
-
-$relPath must be defined. Otherwise an exception is thrown.
-
-If the _fullpath function failed, an exception is thrown with C<eval_error_id> set to the basic type ID of the L<Error::System|SPVM::Error::System> class.
-
 =head2 chdir
 
 C<static method chdir : int ($path : string);>
