@@ -15,6 +15,9 @@ static const char* FILE_NAME = "Sys/IO/Windows.c";
 #include <errno.h>
 #include <winbase.h>
 #include <fcntl.h>
+#include <direct.h>
+
+#include "Sys-Windows.h"
 
 static OSVERSIONINFO g_osver = {0, 0, 0, 0, 0, ""};
 
@@ -70,6 +73,7 @@ typedef struct {
 #define dTHX 
 
 typedef BOOLEAN (__stdcall *pCreateSymbolicLinkA_t)(LPCSTR, LPCSTR, DWORD);
+typedef BOOLEAN (__stdcall *pCreateSymbolicLinkW_t)(LPCWSTR, LPCWSTR, DWORD);
 
 static BOOL
 is_symlink(HANDLE h) {
@@ -92,9 +96,10 @@ is_symlink(HANDLE h) {
     return TRUE;
 }
 
+// Same as Perl's is_symlink_name in Win32.c, but call APIs for wide characters(W instead of A).
 static BOOL
-is_symlink_name(const char *name) {
-    HANDLE f = CreateFileA(name, GENERIC_READ, 0, NULL, OPEN_EXISTING,
+is_symlink_name(const wchar_t *name) {
+    HANDLE f = CreateFileW(name, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                            FILE_FLAG_OPEN_REPARSE_POINT|FILE_FLAG_BACKUP_SEMANTICS, 0);
     BOOL result;
 
@@ -107,50 +112,51 @@ is_symlink_name(const char *name) {
     return result;
 }
 
+// Same as Perl's win32_unlink in Win32.c, but call APIs for wide characters(W instead of A and _w prefixed).
 static int
-win32_unlink(const char *filename)
+win32_unlink(const wchar_t *filename)
 {
   int ret;
   DWORD attrs;
   
-  attrs = GetFileAttributesA(filename);
+  attrs = GetFileAttributesW(filename);
   if (attrs == 0xFFFFFFFF) {
     errno = ENOENT;
     return -1;
   }
   
   if (attrs & FILE_ATTRIBUTE_READONLY) {
-    (void)SetFileAttributesA(filename, attrs & ~FILE_ATTRIBUTE_READONLY);
-    ret = unlink(filename);
+    (void)SetFileAttributesW(filename, attrs & ~FILE_ATTRIBUTE_READONLY);
+    ret = _wunlink(filename);
     if (ret == -1)
-        (void)SetFileAttributesA(filename, attrs);
+        (void)SetFileAttributesW(filename, attrs);
   }
   else if ((attrs & (FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY))
     == (FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY)
          && is_symlink_name(filename)) {
-    ret = rmdir(filename);
+    ret = _wrmdir(filename);
   }
   else {
-    ret = unlink(filename);
+    ret = _wunlink(filename);
   }
   
   return ret;
 }
 
-// Exactly same as Perl's win32_rename in Win32.c
+// Same as Perl's win32_rename in Win32.c, but call APIs for wide characters(W instead of A and _w prefixed).
 static int
-win32_rename(const char *oname, const char *newname)
+win32_rename(const wchar_t *oname, const wchar_t *newname)
 {
-    char szOldName[MAX_PATH+1];
+    wchar_t szOldName[MAX_PATH+1];
     BOOL bResult;
     DWORD dwFlags = MOVEFILE_COPY_ALLOWED;
     dTHX;
 
-    if (stricmp(newname, oname))
+    if (_wcsicmp(newname, oname))
         dwFlags |= MOVEFILE_REPLACE_EXISTING;
-    strcpy(szOldName, PerlDir_mapA(oname));
+    wcscpy(szOldName, oname);
     
-    bResult = MoveFileExA(szOldName,PerlDir_mapA(newname), dwFlags);
+    bResult = MoveFileExW(szOldName,newname, dwFlags);
     
     if (!bResult) {
         DWORD err = GetLastError();
@@ -332,12 +338,15 @@ win32_readlink(const char *pathname, char *buf, size_t bufsiz) {
     return bytes_out;
 }
 
+#define isSLASHW(c) ((c) == L'/' || (c) == L'\\')
+#define strEQW(string1, string2) (wcscmp(string1, string2) == 0)
+
 static int
-win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char *newfile)
+win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const wchar_t *oldfile, const wchar_t *newfile)
 {
-    size_t oldfile_len = strlen(oldfile);
-    pCreateSymbolicLinkA_t pCreateSymbolicLinkA =
-        (pCreateSymbolicLinkA_t)GetProcAddress(GetModuleHandle("kernel32.dll"), "CreateSymbolicLinkA");
+    size_t oldfile_len = wcslen(oldfile);
+    pCreateSymbolicLinkW_t pCreateSymbolicLinkW =
+        (pCreateSymbolicLinkW_t)GetProcAddress(GetModuleHandle("kernel32.dll"), "CreateSymbolicLinkW");
     DWORD create_flags = 0;
 
     /* this flag can be used only on Windows 10 1703 or newer */
@@ -348,19 +357,20 @@ win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char
         create_flags |= SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
     }
 
-    if (!pCreateSymbolicLinkA) {
+    if (!pCreateSymbolicLinkW) {
         errno = ENOSYS;
         return -1;
     }
 
-    if (strchr(oldfile, '/')) {
+    if (wcschr(oldfile, L'/')) {
         /* Win32 (or perhaps NTFS) won't follow symlinks containing
            /, so replace any with \\
         */
-        char *temp = (char*)env->get_chars(env, stack, env->new_string(env, stack, oldfile, strlen(oldfile)));
-        char *p = temp;
+        wchar_t *temp = (wchar_t*)env->new_memory_block(env, stack, (wcslen(oldfile) + 1) * sizeof(wchar_t));
+        memcpy(temp, oldfile, (wcslen(oldfile) + 1) * sizeof(wchar_t));
+        wchar_t *p = temp;
         while (*p) {
-            if (*p == '/') {
+            if (*p == L'/') {
                 *p = '\\';
             }
             ++p;
@@ -368,10 +378,11 @@ win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char
         *p = 0;
         oldfile = temp;
         oldfile_len = p - temp;
+        env->free_memory_block(env, stack, temp);
     }
 
     /* are we linking to a directory?
-       CreateSymlinkA() needs to know if the target is a directory,
+       CreateSymlinkW() needs to know if the target is a directory,
        If it looks like a directory name:
         - ends in slash
         - is just . or ..
@@ -381,28 +392,28 @@ win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char
        Otherwise if the oldfile is relative we need to make a relative path
        based on the newfile to check if the target is a directory.
     */
-    if ((oldfile_len >= 1 && isSLASH(oldfile[oldfile_len-1])) ||
-        strEQ(oldfile, "..") ||
-        strEQ(oldfile, ".") ||
+    if ((oldfile_len >= 1 && isSLASHW(oldfile[oldfile_len-1])) ||
+        strEQW(oldfile, L"..") ||
+        strEQW(oldfile, L".") ||
         (isSLASH(oldfile[oldfile_len-2]) && oldfile[oldfile_len-1] == '.') ||
-        strEQ(oldfile+oldfile_len-3, "\\..") ||
-        (oldfile_len == 2 && oldfile[1] == ':')) {
+        strEQW(oldfile+oldfile_len-3, L"\\..") ||
+        (oldfile_len == 2 && oldfile[1] == L':')) {
         create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
     }
     else {
         DWORD dest_attr;
-        const char *dest_path = oldfile;
-        char szTargetName[MAX_PATH+1];
+        const wchar_t *dest_path = oldfile;
+        wchar_t szTargetName[MAX_PATH+1];
 
         if (oldfile_len >= 3 && oldfile[1] == ':') {
             /* relative to current directory on a drive, or absolute */
             /* dest_path = oldfile; already done */
         }
-        else if (oldfile[0] != '\\') {
-            size_t newfile_len = strlen(newfile);
-            char *last_slash = strrchr(newfile, '/');
-            char *last_bslash = strrchr(newfile, '\\');
-            char *end_dir = last_slash && last_bslash
+        else if (oldfile[0] != L'\\') {
+            size_t newfile_len = wcslen(newfile);
+            wchar_t *last_slash = wcsrchr(newfile, L'/');
+            wchar_t *last_bslash = wcsrchr(newfile, L'\\');
+            wchar_t *end_dir = last_slash && last_bslash
                 ? ( last_slash > last_bslash ? last_slash : last_bslash)
                 : last_slash ? last_slash : last_bslash ? last_bslash : NULL;
 
@@ -413,8 +424,8 @@ win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char
                     return -1;
                 }
 
-                memcpy(szTargetName, newfile, end_dir - newfile + 1);
-                strcpy(szTargetName + (end_dir - newfile + 1), oldfile);
+                memcpy(szTargetName, newfile, (end_dir - newfile + 1) * sizeof(wchar_t));
+                wcscpy(szTargetName + (end_dir - newfile + 1), oldfile);
                 dest_path = szTargetName;
             }
             else {
@@ -423,13 +434,13 @@ win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const char *oldfile, const char
             }
         }
 
-        dest_attr = GetFileAttributes(dest_path);
+        dest_attr = GetFileAttributesW(dest_path);
         if (dest_attr != (DWORD)-1 && (dest_attr & FILE_ATTRIBUTE_DIRECTORY)) {
             create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
         }
     }
 
-    if (!pCreateSymbolicLinkA(newfile, oldfile, create_flags)) {
+    if (!pCreateSymbolicLinkW(newfile, oldfile, create_flags)) {
         translate_to_errno();
         return -1;
     }
@@ -490,10 +501,10 @@ static int win32_lstat(const char* path, struct stat* sbuf)
 }
 
 // Original implementation
-static int win32_realpath(const char* path, char* out_path, int32_t out_path_length) {
+static int win32_realpath(const wchar_t* path, wchar_t* out_path, int32_t out_path_length) {
   
   int32_t len = 0; // 0 indicates an error
-  HANDLE hFile = CreateFileA(path, FILE_READ_ATTRIBUTES,
+  HANDLE hFile = CreateFileW(path, FILE_READ_ATTRIBUTES,
                     FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE,
                     NULL, OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, NULL);
   
@@ -502,7 +513,7 @@ static int win32_realpath(const char* path, char* out_path, int32_t out_path_len
     goto END_OF_FUNC;
   }
   
-  len = GetFinalPathNameByHandleA(hFile, out_path, out_path_length, 0);
+  len = GetFinalPathNameByHandleW(hFile, out_path, out_path_length, 0);
   
   END_OF_FUNC:
   
@@ -518,6 +529,9 @@ int32_t SPVM__Sys__IO__Windows__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   env->die(env, stack, "Sys::IO::Windows#unlink method is not supported in this system(!defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
+  
+  int32_t error_id = 0;
+  
   void* obj_pathname = stack[0].oval;
   if (!obj_pathname) {
     return env->die(env, stack, "The path $pathname must be defined.", __func__, FILE_NAME, __LINE__);
@@ -525,7 +539,12 @@ int32_t SPVM__Sys__IO__Windows__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* pathname = env->get_chars(env, stack, obj_pathname);
   
-  int32_t status = win32_unlink(pathname);
+  wchar_t* pathname_w = utf8_to_win_wchar(env, stack, pathname, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = win32_unlink(pathname_w);
   
   stack[0].ival = status;
   if (status == -1) {
@@ -542,6 +561,9 @@ int32_t SPVM__Sys__IO__Windows__rename(SPVM_ENV* env, SPVM_VALUE* stack) {
   env->die(env, stack, "Sys::IO::Windows#rename method is not supported in this system(!defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
   return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS;
 #else
+  
+  int32_t error_id = 0;
+  
   void* obj_oldpath = stack[0].oval;
   if (!obj_oldpath) {
     return env->die(env, stack, "The old path $oldpath must be defined.", __func__, FILE_NAME, __LINE__);
@@ -554,7 +576,17 @@ int32_t SPVM__Sys__IO__Windows__rename(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* newpath = env->get_chars(env, stack, obj_newpath);
   
-  int32_t status = win32_rename(oldpath, newpath);
+  wchar_t* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  wchar_t* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t status = win32_rename(oldpath_w, newpath_w);
   
   if (status == -1) {
     env->die(env, stack, "[System Error]rename() failed:%s. $oldpath is \"%s\". $newpath is \"%s\".", env->strerror_nolen(env, stack, errno), oldpath, newpath, __func__, FILE_NAME, __LINE__);
@@ -629,8 +661,19 @@ int32_t SPVM__Sys__IO__Windows__symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* newpath = env->get_chars(env, stack, obj_newpath);
   
+  wchar_t* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  wchar_t* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
   errno = 0;
-  int32_t status = win32_symlink(env, stack, oldpath, newpath);
+  int32_t status = win32_symlink(env, stack, oldpath_w, newpath_w);
+  
   if (status == -1) {
     env->die(env, stack, "[System Error]win32_symlink() failed:%s. $oldpath is \"%s\". $newpath is \"%s\".", env->strerror_nolen(env, stack, errno), oldpath, newpath, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -685,6 +728,8 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   return env->die(env, stack, "Sys::IO::Windows#realpath method is not supported in this system(!defined(_WIN32)).", __func__, FILE_NAME, __LINE__);
 #else
   
+  int32_t error_id = 0;
+  
   void* obj_path = stack[0].oval;
   
   void* obj_resolved_path = stack[1].oval;
@@ -699,7 +744,13 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  int32_t len = win32_realpath(path, NULL, 0);
+  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  if (error_id) {
+    return error_id;
+  }
+  
+  int32_t len = win32_realpath(path_w, NULL, 0);
+  
   if (len == -1) {
     env->die(env, stack, "[System Error]CreateFile() failed:the symbolic link is not permitted, broken or not found. $path:\"%s\".", path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -709,29 +760,39 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
-  obj_resolved_path = env->new_string(env, stack, NULL, len);
-  char* resolved_path = (char*)env->get_chars(env, stack, obj_resolved_path);
+  void* obj_resolved_path_w = env->new_short_array(env, stack, len + 1);
+  wchar_t* resolved_path_w = (wchar_t*)env->get_elems_short(env, stack, obj_resolved_path_w);
+  
+  len = win32_realpath(path_w, resolved_path_w, len);
   
-  len = win32_realpath(path, resolved_path, len + 1);
   if (!(len > 0)) {
     env->die(env, stack, "[System Error]win32_realpath() failed:GetLastError() %d. $path:\"%s\".", GetLastError(), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
+  const char* resolved_path_tmp = win_wchar_to_utf8(env, stack, resolved_path_w, &error_id, __func__, FILE_NAME, __LINE__);
+  
+  if (error_id) {
+    return error_id;
+  }
+  obj_resolved_path = env->new_string(env, stack, resolved_path_tmp, strlen(resolved_path_tmp));
+  char* resolved_path = (char*)env->get_chars(env, stack, obj_resolved_path);
+  int32_t resolved_path_length = env->length(env, stack, obj_resolved_path);
+  
   if (strncmp(resolved_path, "\\\\?\\", 4) == 0) {
-    for (int32_t i = 0; i < len - 4; i++) {
+    for (int32_t i = 0; i < resolved_path_length - 4; i++) {
       resolved_path[i] = resolved_path[i + 4];
     }
-    env->shorten(env, stack, obj_resolved_path, len - 4);
+    env->shorten(env, stack, obj_resolved_path, resolved_path_length - 4);
   }
   else if (strncmp(resolved_path, "\\\\?\\UNC\\", 8) == 0) {
-    for (int32_t i = 2; i < len - 6; i++) {
+    for (int32_t i = 2; i < resolved_path_length - 6; i++) {
       resolved_path[i] = resolved_path[i + 6];
     }
-    env->shorten(env, stack, obj_resolved_path, len - 8);
+    env->shorten(env, stack, obj_resolved_path, resolved_path_length - 8);
   }
   
-  for (int32_t i = 0; i < len; i++) {
+  for (int32_t i = 0; i < resolved_path_length; i++) {
     if (resolved_path[i] == '\\') {
       resolved_path[i] = '/';
     }
